@@ -2,7 +2,7 @@
  *
  * syslogger.c
  *
- * The system logger (syslogger) appeared in Postgres 8.0. It catches all
+ * The system logger (syslogger) is new in Postgres 8.0. It catches all
  * stderr output from the postmaster, backends, and other subprocesses
  * by redirecting to a pipe, and writes it to a set of logfiles.
  * It's possible to have size and age limits for the logfile configured
@@ -14,11 +14,11 @@
  *
  * Author: Andreas Pflug <pgadmin@pse-consulting.de>
  *
- * Copyright (c) 2004-2009, PostgreSQL Global Development Group
+ * Copyright (c) 2004-2010, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/postmaster/syslogger.c,v 1.51.2.3 2010/04/16 09:51:54 heikki Exp $
+ *	  $PostgreSQL: pgsql/src/backend/postmaster/syslogger.c,v 1.58 2010/07/06 19:18:57 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,6 +45,17 @@
 #include "utils/ps_status.h"
 #include "utils/timestamp.h"
 
+/*
+ * We really want line-buffered mode for logfile output, but Windows does
+ * not have it, and interprets _IOLBF as _IOFBF (bozos).  So use _IONBF
+ * instead on Windows.
+ */
+#ifdef WIN32
+#define LBF_MODE	_IONBF
+#else
+#define LBF_MODE	_IOLBF
+#endif
+
 /*
  * We read() into a temp buffer twice as big as a chunk, so that any fragment
  * left after processing can be moved down to the front and we'll still have
@@ -54,7 +65,7 @@
 
 
 /*
- * GUC parameters.  Logging_collector cannot be changed after postmaster
+ * GUC parameters.	Logging_collector cannot be changed after postmaster
  * start, but the rest can change at SIGHUP.
  */
 bool		Logging_collector = false;
@@ -79,7 +90,6 @@ static bool pipe_eof_seen = false;
 static bool rotation_disabled = false;
 static FILE *syslogFile = NULL;
 static FILE *csvlogFile = NULL;
-NON_EXEC_STATIC pg_time_t first_syslogger_file_time = 0;
 static char *last_file_name = NULL;
 static char *last_csv_file_name = NULL;
 
@@ -173,7 +183,7 @@ SysLoggerMain(int argc, char *argv[])
 	/*
 	 * If we restarted, our stderr is already redirected into our own input
 	 * pipe.  This is of course pretty useless, not to mention that it
-	 * interferes with detecting pipe EOF.  Point stderr to /dev/null. This
+	 * interferes with detecting pipe EOF.	Point stderr to /dev/null. This
 	 * assumes that all interesting messages generated in the syslogger will
 	 * come through elog.c and will be sent to write_syslogger_file.
 	 */
@@ -183,7 +193,7 @@ SysLoggerMain(int argc, char *argv[])
 
 		/*
 		 * The closes might look redundant, but they are not: we want to be
-		 * darn sure the pipe gets closed even if the open failed.  We can
+		 * darn sure the pipe gets closed even if the open failed.	We can
 		 * survive running with stderr pointing nowhere, but we can't afford
 		 * to have extra pipe input descriptors hanging around.
 		 */
@@ -224,7 +234,7 @@ SysLoggerMain(int argc, char *argv[])
 
 	/*
 	 * If possible, make this process a group leader, so that the postmaster
-	 * can signal any child processes too.  (syslogger probably never has any
+	 * can signal any child processes too.	(syslogger probably never has any
 	 * child processes, but for consistency we make all postmaster child
 	 * processes do this.)
 	 */
@@ -271,13 +281,6 @@ SysLoggerMain(int argc, char *argv[])
 		elog(FATAL, "could not create syslogger data transfer thread: %m");
 #endif   /* WIN32 */
 
-	/*
-	 * Remember active logfile's name.  We recompute this from the reference
-	 * time because passing down just the pg_time_t is a lot cheaper than
-	 * passing a whole file path in the EXEC_BACKEND case.
-	 */
-	last_file_name = logfile_getname(first_syslogger_file_time, NULL);
-
 	/* remember active logfile parameters */
 	currentLogDir = pstrdup(Log_directory);
 	currentLogFilename = pstrdup(Log_filename);
@@ -389,7 +392,7 @@ SysLoggerMain(int argc, char *argv[])
 		 * Wait for some data, timing out after 1 second
 		 */
 		FD_ZERO(&rfds);
-		FD_SET		(syslogPipe[0], &rfds);
+		FD_SET(syslogPipe[0], &rfds);
 
 		timeout.tv_sec = 1;
 		timeout.tv_usec = 0;
@@ -442,9 +445,9 @@ SysLoggerMain(int argc, char *argv[])
 		 * detect pipe EOF.  The main thread just wakes up once a second to
 		 * check for SIGHUP and rotation conditions.
 		 *
-		 * Server code isn't generally thread-safe, so we ensure that only
-		 * one of the threads is active at a time by entering the critical
-		 * section whenever we're not sleeping.
+		 * Server code isn't generally thread-safe, so we ensure that only one
+		 * of the threads is active at a time by entering the critical section
+		 * whenever we're not sleeping.
 		 */
 		LeaveCriticalSection(&sysloggerSection);
 
@@ -463,7 +466,7 @@ SysLoggerMain(int argc, char *argv[])
 					(errmsg("logger shutting down")));
 
 			/*
-			 * Normal exit from the syslogger is here.  Note that we
+			 * Normal exit from the syslogger is here.	Note that we
 			 * deliberately do not close syslogFile before exiting; this is to
 			 * allow for the possibility of elog messages being generated
 			 * inside proc_exit.  Regular exit() will take care of flushing
@@ -529,18 +532,9 @@ SysLogger_Start(void)
 
 	/*
 	 * The initial logfile is created right in the postmaster, to verify that
-	 * the Log_directory is writable.  We save the reference time so that
-	 * the syslogger child process can recompute this file name.
-	 *
-	 * It might look a bit strange to re-do this during a syslogger restart,
-	 * but we must do so since the postmaster closed syslogFile after the
-	 * previous fork (and remembering that old file wouldn't be right anyway).
-	 * Note we always append here, we won't overwrite any existing file.  This
-	 * is consistent with the normal rules, because by definition this is not
-	 * a time-based rotation.
+	 * the Log_directory is writable.
 	 */
-	first_syslogger_file_time = time(NULL);
-	filename = logfile_getname(first_syslogger_file_time, NULL);
+	filename = logfile_getname(time(NULL), NULL);
 
 	syslogFile = fopen(filename, "a");
 
@@ -550,7 +544,7 @@ SysLogger_Start(void)
 				 (errmsg("could not create log file \"%s\": %m",
 						 filename))));
 
-	setvbuf(syslogFile, NULL, PG_IOLBF, 0);
+	setvbuf(syslogFile, NULL, LBF_MODE, 0);
 
 	pfree(filename);
 
@@ -611,7 +605,7 @@ SysLogger_Start(void)
 				 * chunking protocol.
 				 */
 				fflush(stderr);
-				fd = _open_osfhandle((long) syslogPipe[1],
+				fd = _open_osfhandle((intptr_t) syslogPipe[1],
 									 _O_APPEND | _O_BINARY);
 				if (dup2(fd, _fileno(stderr)) < 0)
 					ereport(FATAL,
@@ -698,7 +692,7 @@ syslogger_parseArgs(int argc, char *argv[])
 	if (fd != -1)
 	{
 		syslogFile = fdopen(fd, "a");
-		setvbuf(syslogFile, NULL, PG_IOLBF, 0);
+		setvbuf(syslogFile, NULL, LBF_MODE, 0);
 	}
 #else							/* WIN32 */
 	fd = atoi(*argv++);
@@ -708,7 +702,7 @@ syslogger_parseArgs(int argc, char *argv[])
 		if (fd > 0)
 		{
 			syslogFile = fdopen(fd, "a");
-			setvbuf(syslogFile, NULL, PG_IOLBF, 0);
+			setvbuf(syslogFile, NULL, LBF_MODE, 0);
 		}
 	}
 #endif   /* WIN32 */
@@ -1028,12 +1022,8 @@ pipeThread(void *arg)
 #endif   /* WIN32 */
 
 /*
- * Open the csv log file - we do this opportunistically, because
+ * open the csv log file - we do this opportunistically, because
  * we don't know if CSV logging will be wanted.
- *
- * This is only used the first time we open the csv log in a given syslogger
- * process, not during rotations.  As with opening the main log file, we
- * always append in this situation.
  */
 static void
 open_csvlogfile(void)
@@ -1051,7 +1041,7 @@ open_csvlogfile(void)
 				 (errmsg("could not create log file \"%s\": %m",
 						 filename))));
 
-	setvbuf(fh, NULL, PG_IOLBF, 0);
+	setvbuf(fh, NULL, LBF_MODE, 0);
 
 #ifdef WIN32
 	_setmode(_fileno(fh), _O_TEXT);		/* use CRLF line endings on Windows */
@@ -1059,10 +1049,8 @@ open_csvlogfile(void)
 
 	csvlogFile = fh;
 
-	if (last_csv_file_name != NULL)		/* probably shouldn't happen */
-		pfree(last_csv_file_name);
+	pfree(filename);
 
-	last_csv_file_name = filename;
 }
 
 /*
@@ -1097,7 +1085,14 @@ logfile_rotate(bool time_based_rotation, int size_rotation_for)
 	 * elapsed time and not something else, and (c) the computed file name is
 	 * different from what we were previously logging into.
 	 *
-	 * Note: last_file_name should never be NULL here, but if it is, append.
+	 * Note: during the first rotation after forking off from the postmaster,
+	 * last_file_name will be NULL.  (We don't bother to set it in the
+	 * postmaster because it ain't gonna work in the EXEC_BACKEND case.) So we
+	 * will always append in that situation, even though truncating would
+	 * usually be safe.
+	 *
+	 * For consistency, we treat CSV logs the same even though they aren't
+	 * opened in the postmaster.
 	 */
 	if (time_based_rotation || (size_rotation_for & LOG_DESTINATION_STDERR))
 	{
@@ -1126,7 +1121,7 @@ logfile_rotate(bool time_based_rotation, int size_rotation_for)
 			if (saveerrno != ENFILE && saveerrno != EMFILE)
 			{
 				ereport(LOG,
-						(errmsg("disabling automatic rotation (use SIGHUP to reenable)")));
+						(errmsg("disabling automatic rotation (use SIGHUP to re-enable)")));
 				rotation_disabled = true;
 			}
 
@@ -1137,7 +1132,7 @@ logfile_rotate(bool time_based_rotation, int size_rotation_for)
 			return;
 		}
 
-		setvbuf(fh, NULL, PG_IOLBF, 0);
+		setvbuf(fh, NULL, LBF_MODE, 0);
 
 #ifdef WIN32
 		_setmode(_fileno(fh), _O_TEXT); /* use CRLF line endings on Windows */
@@ -1183,7 +1178,7 @@ logfile_rotate(bool time_based_rotation, int size_rotation_for)
 			if (saveerrno != ENFILE && saveerrno != EMFILE)
 			{
 				ereport(LOG,
-						(errmsg("disabling automatic rotation (use SIGHUP to reenable)")));
+						(errmsg("disabling automatic rotation (use SIGHUP to re-enable)")));
 				rotation_disabled = true;
 			}
 
@@ -1194,7 +1189,7 @@ logfile_rotate(bool time_based_rotation, int size_rotation_for)
 			return;
 		}
 
-		setvbuf(fh, NULL, PG_IOLBF, 0);
+		setvbuf(fh, NULL, LBF_MODE, 0);
 
 #ifdef WIN32
 		_setmode(_fileno(fh), _O_TEXT); /* use CRLF line endings on Windows */
@@ -1271,7 +1266,7 @@ set_next_rotation_time(void)
 	/*
 	 * The requirements here are to choose the next time > now that is a
 	 * "multiple" of the log rotation interval.  "Multiple" can be interpreted
-	 * fairly loosely.  In this version we align to log_timezone rather than
+	 * fairly loosely.	In this version we align to log_timezone rather than
 	 * GMT.
 	 */
 	rotinterval = Log_RotationAge * SECS_PER_MINUTE;	/* convert to seconds */
