@@ -1,8 +1,8 @@
 #! /usr/bin/perl
 #
-# Copyright (c) 2001-2005, PostgreSQL Global Development Group
+# Copyright (c) 2001-2010, PostgreSQL Global Development Group
 #
-# $PostgreSQL: pgsql/src/backend/utils/mb/Unicode/UCS_to_EUC_KR.pl,v 1.7 2005/01/01 20:44:18 tgl Exp $
+# $PostgreSQL: pgsql/src/backend/utils/mb/Unicode/UCS_to_EUC_KR.pl,v 1.13 2010/01/02 16:57:56 momjian Exp $
 #
 # Generate UTF-8 <--> EUC_KR code conversion tables from
 # map files provided by Unicode organization.
@@ -35,7 +35,7 @@ while( <FILE> ){
 	if( $code >= 0x80 && $ucs >= 0x0080 ){
 		$utf = &ucs2utf($ucs);
 		if( $array{ $utf } ne "" ){
-			printf STDERR "Warning: duplicate unicode: %04x\n",$ucs;
+			printf STDERR "Warning: duplicate UTF8: %04x\n",$ucs;
 			next;
 		}
 		$count++;
