@@ -3,10 +3,10 @@
  * pgtz.c
  *	  Timezone Library Integration Functions
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/timezone/pgtz.c,v 1.63.2.5 2010/05/20 14:13:23 mha Exp $
+ *	  $PostgreSQL: pgsql/src/timezone/pgtz.c,v 1.74 2010/07/06 19:19:01 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -94,7 +94,7 @@ pg_open_tzfile(const char *name, char *canonname)
 	 * Loop to split the given name into directory levels; for each level,
 	 * search using scan_directory_ci().
 	 */
-	strlcpy(fullname, pg_TZDIR(), sizeof(fullname));
+	strcpy(fullname, pg_TZDIR());
 	orignamelen = fullnamelen = strlen(fullname);
 	fname = name;
 	for (;;)
@@ -428,7 +428,7 @@ identify_system_timezone(void)
 	}
 
 	/* Search for the best-matching timezone file */
-	strlcpy(tmptzdir, pg_TZDIR(), sizeof(tmptzdir));
+	strcpy(tmptzdir, pg_TZDIR());
 	bestscore = -1;
 	resultbuf[0] = '\0';
 	scan_available_timezones(tmptzdir, tmptzdir + strlen(tmptzdir) + 1,
@@ -498,7 +498,9 @@ identify_system_timezone(void)
 	if (std_zone_name[0] == '\0')
 	{
 		ereport(LOG,
-				(errmsg("could not determine system time zone, defaulting to \"%s\"", "GMT"),
+				(errmsg("could not determine system time zone"),
+				 errdetail("The PostgreSQL time zone will be set to \"%s\".",
+						   "GMT"),
 		errhint("You can specify the correct timezone in postgresql.conf.")));
 		return NULL;			/* go to GMT */
 	}
@@ -533,8 +535,9 @@ identify_system_timezone(void)
 			 (-std_ofs > 0) ? "+" : "", -std_ofs / 3600);
 
 	ereport(LOG,
-		 (errmsg("could not recognize system timezone, defaulting to \"%s\"",
-				 resultbuf),
+			(errmsg("could not recognize system time zone"),
+			 errdetail("The PostgreSQL time zone will be set to \"%s\".",
+					   resultbuf),
 	   errhint("You can specify the correct timezone in postgresql.conf.")));
 	return resultbuf;
 }
@@ -1035,9 +1038,13 @@ static const struct
 		"W. Australia Standard Time", "W. Australia Daylight Time",
 		"Australia/Perth"
 	},							/* (GMT+08:00) Perth */
-/*	{"W. Central Africa Standard Time", "W. Central Africa Daylight Time",
-	 *	 *	 *	 *	 *	 *	 *	 *	""}, Could not find a match for this one. Excluded for now. *//* (
-	 * G MT+01:00) West Central Africa */
+#ifdef NOT_USED
+	/* Could not find a match for this one (just a guess). Excluded for now. */
+	{
+		"W. Central Africa Standard Time", "W. Central Africa Daylight Time",
+		"WAT"
+	},							/* (GMT+01:00) West Central Africa */
+#endif
 	{
 		"W. Europe Standard Time", "W. Europe Daylight Time",
 		"CET"
@@ -1073,9 +1080,12 @@ identify_system_timezone(void)
 
 	if (!tm)
 	{
-		ereport(WARNING,
-				(errmsg_internal("could not determine current date/time: localtime failed")));
-		return NULL;
+		ereport(LOG,
+		  (errmsg("could not identify system time zone: localtime() failed"),
+		   errdetail("The PostgreSQL time zone will be set to \"%s\".",
+					 "GMT"),
+		errhint("You can specify the correct timezone in postgresql.conf.")));
+		return NULL;			/* go to GMT */
 	}
 
 	memset(tzname, 0, sizeof(tzname));
@@ -1086,7 +1096,7 @@ identify_system_timezone(void)
 		if (strcmp(tzname, win32_tzmap[i].stdname) == 0 ||
 			strcmp(tzname, win32_tzmap[i].dstname) == 0)
 		{
-			elog(DEBUG4, "TZ \"%s\" matches Windows timezone \"%s\"",
+			elog(DEBUG4, "TZ \"%s\" matches system time zone \"%s\"",
 				 win32_tzmap[i].pgtzname, tzname);
 			return win32_tzmap[i].pgtzname;
 		}
@@ -1104,9 +1114,13 @@ identify_system_timezone(void)
 					 KEY_READ,
 					 &rootKey) != ERROR_SUCCESS)
 	{
-		ereport(WARNING,
-				(errmsg_internal("could not open registry key to identify Windows timezone: %i", (int) GetLastError())));
-		return NULL;
+		ereport(LOG,
+				(errmsg("could not open registry key to identify system time zone: %i",
+						(int) GetLastError()),
+				 errdetail("The PostgreSQL time zone will be set to \"%s\".",
+						   "GMT"),
+		errhint("You can specify the correct timezone in postgresql.conf.")));
+		return NULL;			/* go to GMT */
 	}
 
 	for (idx = 0;; idx++)
@@ -1131,15 +1145,15 @@ identify_system_timezone(void)
 		{
 			if (r == ERROR_NO_MORE_ITEMS)
 				break;
-			ereport(WARNING,
-					(errmsg_internal("could not enumerate registry subkeys to identify Windows timezone: %i", (int) r)));
+			ereport(LOG,
+					(errmsg_internal("could not enumerate registry subkeys to identify system time zone: %i", (int) r)));
 			break;
 		}
 
 		if ((r = RegOpenKeyEx(rootKey, keyname, 0, KEY_READ, &key)) != ERROR_SUCCESS)
 		{
-			ereport(WARNING,
-					(errmsg_internal("could not open registry subkey to identify Windows timezone: %i", (int) r)));
+			ereport(LOG,
+					(errmsg_internal("could not open registry subkey to identify system time zone: %i", (int) r)));
 			break;
 		}
 
@@ -1147,11 +1161,11 @@ identify_system_timezone(void)
 		namesize = sizeof(zonename);
 		if ((r = RegQueryValueEx(key, "Std", NULL, NULL, zonename, &namesize)) != ERROR_SUCCESS)
 		{
-			ereport(WARNING,
-					(errmsg_internal("could not query value for 'std' to identify Windows timezone \"%s\": %i",
+			ereport(LOG,
+					(errmsg_internal("could not query value for key \"std\" to identify system time zone \"%s\": %i",
 									 keyname, (int) r)));
 			RegCloseKey(key);
-			continue; /* Proceed to look at the next timezone */
+			continue;			/* Proceed to look at the next timezone */
 		}
 		if (strcmp(tzname, zonename) == 0)
 		{
@@ -1164,11 +1178,11 @@ identify_system_timezone(void)
 		namesize = sizeof(zonename);
 		if ((r = RegQueryValueEx(key, "Dlt", NULL, NULL, zonename, &namesize)) != ERROR_SUCCESS)
 		{
-			ereport(WARNING,
-					(errmsg_internal("could not query value for 'dlt' to identify Windows timezone \"%s\": %i",
+			ereport(LOG,
+					(errmsg_internal("could not query value for key \"dlt\" to identify system time zone \"%s\": %i",
 									 keyname, (int) r)));
 			RegCloseKey(key);
-			continue; /* Proceed to look at the next timezone */
+			continue;			/* Proceed to look at the next timezone */
 		}
 		if (strcmp(tzname, zonename) == 0)
 		{
@@ -1191,17 +1205,20 @@ identify_system_timezone(void)
 			if (strcmp(localtzname, win32_tzmap[i].stdname) == 0 ||
 				strcmp(localtzname, win32_tzmap[i].dstname) == 0)
 			{
-				elog(DEBUG4, "TZ \"%s\" matches localized Windows timezone \"%s\" (\"%s\")",
+				elog(DEBUG4, "TZ \"%s\" matches localized system time zone \"%s\" (\"%s\")",
 					 win32_tzmap[i].pgtzname, tzname, localtzname);
 				return win32_tzmap[i].pgtzname;
 			}
 		}
 	}
 
-	ereport(WARNING,
-			(errmsg("could not find a match for Windows timezone \"%s\"",
-					tzname)));
-	return NULL;
+	ereport(LOG,
+			(errmsg("could not find a match for system time zone \"%s\"",
+					tzname),
+			 errdetail("The PostgreSQL time zone will be set to \"%s\".",
+					   "GMT"),
+	   errhint("You can specify the correct timezone in postgresql.conf.")));
+	return NULL;				/* go to GMT */
 }
 #endif   /* WIN32 */
 
@@ -1308,46 +1325,6 @@ pg_tzset(const char *name)
 	return &tzp->tz;
 }
 
-/*
- * Load a fixed-GMT-offset timezone.
- * This is used for SQL-spec SET TIME ZONE INTERVAL 'foo' cases.
- * It's otherwise equivalent to pg_tzset().
- *
- * The GMT offset is specified in seconds, positive values meaning west of
- * Greenwich (ie, POSIX not ISO sign convention).  However, we use ISO
- * sign convention in the displayable abbreviation for the zone.
- */
-pg_tz *
-pg_tzset_offset(long gmtoffset)
-{
-	long		absoffset = (gmtoffset < 0) ? -gmtoffset : gmtoffset;
-	char		offsetstr[64];
-	char		tzname[128];
-
-	snprintf(offsetstr, sizeof(offsetstr),
-			 "%02ld", absoffset / SECSPERHOUR);
-	absoffset %= SECSPERHOUR;
-	if (absoffset != 0)
-	{
-		snprintf(offsetstr + strlen(offsetstr),
-				 sizeof(offsetstr) - strlen(offsetstr),
-				 ":%02ld", absoffset / SECSPERMIN);
-		absoffset %= SECSPERMIN;
-		if (absoffset != 0)
-			snprintf(offsetstr + strlen(offsetstr),
-					 sizeof(offsetstr) - strlen(offsetstr),
-					 ":%02ld", absoffset);
-	}
-	if (gmtoffset > 0)
-		snprintf(tzname, sizeof(tzname), "<-%s>+%s",
-				 offsetstr, offsetstr);
-	else
-		snprintf(tzname, sizeof(tzname), "<+%s>-%s",
-				 offsetstr, offsetstr);
-
-	return pg_tzset(tzname);
-}
-
 
 /*
  * Check whether timezone is acceptable.
@@ -1427,7 +1404,7 @@ select_default_timezone(void)
 		return def_tz;
 
 	ereport(FATAL,
-			(errmsg("could not select a suitable default timezone"),
+			(errmsg("could not select a suitable default time zone"),
 			 errdetail("It appears that your GMT time zone uses leap seconds. PostgreSQL does not support leap seconds.")));
 	return NULL;				/* keep compiler quiet */
 }
@@ -1451,7 +1428,7 @@ pg_timezone_pre_initialize(void)
 	 * seems OK to just use the "lastditch" case provided by tzparse().
 	 */
 	if (tzparse("GMT", &gmt_timezone_data.state, TRUE) != 0)
-		elog(FATAL, "could not initialize GMT timezone");
+		elog(FATAL, "could not initialize GMT time zone");
 	strcpy(gmt_timezone_data.TZname, "GMT");
 	gmt_timezone = &gmt_timezone_data;
 }
@@ -1469,7 +1446,7 @@ pg_timezone_initialize(void)
 	pg_tz	   *def_tz = NULL;
 
 	/* Do we need to try to figure the session timezone? */
-	if (pg_strcasecmp(GetConfigOption("timezone"), "UNKNOWN") == 0)
+	if (pg_strcasecmp(GetConfigOption("timezone", false), "UNKNOWN") == 0)
 	{
 		/* Select setting */
 		def_tz = select_default_timezone();
@@ -1480,7 +1457,7 @@ pg_timezone_initialize(void)
 	}
 
 	/* What about the log timezone? */
-	if (pg_strcasecmp(GetConfigOption("log_timezone"), "UNKNOWN") == 0)
+	if (pg_strcasecmp(GetConfigOption("log_timezone", false), "UNKNOWN") == 0)
 	{
 		/* Select setting, but don't duplicate work */
 		if (!def_tz)
@@ -1578,7 +1555,7 @@ pg_tzenumerate_next(pg_tzenum *dir)
 			/* Step into the subdirectory */
 			if (dir->depth >= MAX_TZDIR_DEPTH - 1)
 				ereport(ERROR,
-						(errmsg("timezone directory stack overflow")));
+					 (errmsg_internal("timezone directory stack overflow")));
 			dir->depth++;
 			dir->dirname[dir->depth] = pstrdup(fullname);
 			dir->dirdesc[dir->depth] = AllocateDir(fullname);
